import 'dart:collection';
import 'vector.dart';

class Line extends IterableBase<Vector> {
  final Vector start;
  final Vector end;

  const Line(this.start, this.end);

  Line.fromPoints(int x1, int y1, int x2, int y2)
      : start = Vector(x1, y2),
        end = Vector(x2, y2);

  Iterator<Vector> get iterator => _LineIterator(this);
}

class _LineIterator implements Iterator<Vector> {
  final Line _line;
  Vector _current;

  Vector get current => _current;

  int _error;
  int _primary;
  int _secondary;

  Vector _primaryDirection;
  Vector _secondaryDirection;

  _LineIterator(this._line) {
    Vector delta = _line.end - _line.start;

    _primaryDirection = Vector(delta.x.sign, 0);
    _secondaryDirection = Vector(0, delta.y.sign);

    delta = delta.abs();

    _primary = delta.x;
    _secondary = delta.y;

    //If the line travel more rapidly along the y-axis
    //than along the x-axis then swap everything
    if (delta.y > delta.x) {
      Vector temp = _primaryDirection;
      _primaryDirection = _secondaryDirection;
      _secondaryDirection = temp;

      int t = _primary;
      _primary = _secondary;
      _secondary = t;
    }

    _current = _line.start;
    _error = 0;
  }

  ///Always return `true` so that this line
  ///can potentially continue towards infinity.
  ///Stop the iteration manually.
  bool moveNext() {
    //Take a step along the main direction
    _current += _primaryDirection;

    //Should we also move along the secondary direction?
    _error += _secondary;
    if (_error * 2 >= _primary) {
      //If so
      _current += _secondaryDirection; //Take a step in that direction
      _error -= _primary; //Decrease the error
    }

    return true;
  }
}
